pub(crate) mod consts;
pub(crate) mod server;
pub(crate) mod worker;

use std::net::ToSocketAddrs;

use libvanity::{
    vanity::vanity_service_server::VanityServiceServer, worker::worker_server::WorkerServer,
};

pub use libvanity;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    tracing_subscriber::FmtSubscriber::builder()
        .with_max_level(tracing::Level::DEBUG)
        .init();

    let redis = redis::Client::open("redis://127.0.0.1:6379")?;

    let vanity_server = server::VanityService::new(redis.clone());
    let worker_server = worker::WorkerService::new(redis.clone());

    tonic::transport::Server::builder()
        .add_service(VanityServiceServer::new(vanity_server))
        .add_service(WorkerServer::new(worker_server))
        .serve("127.0.0.1:8877".to_socket_addrs()?.next().unwrap())
        .await?;

    Ok(())
}
