use libvanity::vanity::{vanity_service_server as service_server, VanityRequest, VanityResponse};
use tokio_stream::StreamExt;
use tonic::async_trait;

use crate::consts::*;

use redis::AsyncCommands;

pub(crate) struct VanityService {
    counter: std::sync::atomic::AtomicU64,
    redis: redis::Client,
}

impl VanityService {
    pub fn new(redis: redis::Client) -> Self {
        Self {
            redis,
            counter: std::sync::atomic::AtomicU64::new(0),
        }
    }
}

#[async_trait]
impl service_server::VanityService for VanityService {
    async fn request_vanity(
        &self,
        req: tonic::Request<VanityRequest>,
    ) -> Result<tonic::Response<VanityResponse>, tonic::Status> {
        let id = self
            .counter
            .fetch_add(1, std::sync::atomic::Ordering::Relaxed);
        let pattern = req.into_inner().pattern;

        let mut redis = self.redis.get_async_connection().await.expect("msg");
        redis
            .publish::<&str, String, ()>("vanity:new", format!("{}|{}", id, pattern))
            .await
            .expect("should not fail");

        let mut pubsub = redis.into_pubsub();
        pubsub
            .subscribe(format!("{}:{}", REDIS_PUBSUB_PATTERN_RESULT, id))
            .await
            .expect("msg");

        let mut msgs = pubsub.on_message();

        loop {
            if let Some(msg) = msgs.next().await {
                let mut response = VanityResponse::default();
                response.result = String::from_utf8_lossy(msg.get_payload_bytes()).to_string();
                return Ok(tonic::Response::new(response));
            }
        }
    }
}
